/**
 *******************************************************************************
 * @file    tmpm461_exb.c
 * @brief   This file provides API functions for EXB driver.
 * @version V2.0.2.2
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm461_exb.h"


#if defined(__TMPM461_EXB_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup EXB 
  * @brief EXB driver modules
  * @{
  */
/** @defgroup EXB_Private_Defines
  * @{
  */
#define MODE_EXBWAIT_MASK          ((uint32_t)0x00000001)
#define EXB_AS_FIXED_BITS          ((uint32_t)0x60000000)
#define EXB_CLKCTL_CLKDIV_CLEAR    ((uint32_t)0xFFFFFFF9)
#define EXB_CLKCTL_CLKEN_CLEAR     ((uint32_t)0xFFFFFFFE)
#define EXB_CLKCTL_CLKEN_SET       ((uint32_t)0x00000001)

/** @} */
/* End of group EXB_Private_Defines */

/** @defgroup EXB_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group EXB_Private_FunctionPrototypes */

/** @defgroup EXB_Private_Functions
  * @{
  */

/** @} */
/* End of group group EXB_Private_Functions */

/** @defgroup EXB_Exported_Functions
  * @{
  */

/**
  * @brief  Set EXB bus mode
  * @param  BusMode: Select EXB mode.
  *   This parameter can be: EXB_BUS_MULTIPLEX or EXB_BUS_SEPARATE.
  * @retval None
  */
void EXB_SetBusMode(uint8_t BusMode)
{
    /* Check the parameters */
    assert_param(IS_EXB_BUS_MODE(BusMode));

    /* Set EXBMOD<EXBSEL> to select bus mode. */
    if (BusMode == EXB_BUS_MULTIPLEX) {
        TSB_EXB_MOD_EXBSEL = 0U;
    } else {
        TSB_EXB_MOD_EXBSEL = 1U;
    }
}

/**
  * @brief  Set the setup, wait and recovery of the bus cycle to be double or quadruple
  * @param  Cycle: Set the bus cycle to be double or quadruple.
  *   This parameter can be:
  *   EXB_CYCLE_NONE, EXB_CYCLE_DOUBLE or EXB_CYCLE_QUADRUPLE
  * @retval None
  */
void EXB_SetBusCycleExtension(uint8_t Cycle)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_EXB_BUS_CYCLE_EXTENSION(Cycle));

    tmp = TSB_EXB->MOD;
    tmp &= MODE_EXBWAIT_MASK;
    tmp |= (uint32_t) Cycle;
    /* Set EXBMOD<EXBWAIT1:0> to set bus cycle extension. */
    TSB_EXB->MOD = tmp;
}

/**
  * @brief  Enable the specified chip
  * @param  ChipSelect: Select Chip
  *   This parameter can be one of the following values:
  *   EXB_CS0, EXB_CS1,EXB_CS2, EXB_CS3
  * @retval None
  */
void EXB_Enable(uint8_t ChipSelect)
{
    /* Check the parameters */
    assert_param(IS_EXB_CHIP_SELECT(ChipSelect));

    switch (ChipSelect) {
    case EXB_CS0:
        TSB_EXB_CS0_CSW0 = 1U;
        break;

    case EXB_CS1:
        TSB_EXB_CS1_CSW0 = 1U;
        break;

    case EXB_CS2:
        TSB_EXB_CS2_CSW0 = 1U;
        break;

    case EXB_CS3:
        TSB_EXB_CS3_CSW0 = 1U;
        break;

    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Disable the specified chip
  * @param  ChipSelect: Select Chip
  *   This parameter can be one of the following values:
  *   EXB_CS0, EXB_CS1,EXB_CS2, EXB_CS3
  * @retval None
  */
void EXB_Disable(uint8_t ChipSelect)
{
    /* Check the parameters */
    assert_param(IS_EXB_CHIP_SELECT(ChipSelect));

    switch (ChipSelect) {
    case EXB_CS0:
        TSB_EXB_CS0_CSW0 = 0U;
        break;

    case EXB_CS1:
        TSB_EXB_CS1_CSW0 = 0U;
        break;

    case EXB_CS2:
        TSB_EXB_CS2_CSW0 = 0U;
        break;

    case EXB_CS3:
        TSB_EXB_CS3_CSW0 = 0U;
        break;

    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Initialize the specified chip
  * @param  ChipSelect: Select Chip
  *   This parameter can be one of the following values:
  *   EXB_CS0, EXB_CS1,EXB_CS2, EXB_CS3
  * @param  InitStruct: The structure containing basic EXB configuration.
  *   This parameter is structure pointer which consists of following members:
  *   AddrSpaceSize, StartAddr, BusWidth , WaitSignal, WaitFunction and Cycles
  * @retval None
  */
void EXB_Init(uint8_t ChipSelect, EXB_InitTypeDef * InitStruct)
{
    uint32_t tmp_AS = 0U;
    uint32_t tmp_CS = 0U;

    /* Check the parameters */
    assert_param(IS_EXB_CHIP_SELECT(ChipSelect));
    assert_param(IS_POINTER_NOT_NULL(InitStruct));
    assert_param(IS_EXB_ADDR_SPACE_SIZE(InitStruct->AddrSpaceSize));
    assert_param(IS_EXB_DATA_BUS_WIDTH(InitStruct->BusWidth));
    assert_param(IS_EXB_WAIT_SIGNAL(InitStruct->WaitSignal));
    assert_param(IS_EXB_WAIT_FUNCTION(InitStruct->WaitFunction));
    if (InitStruct->WaitFunction == EXB_WAIT_FUNCTION_INT) {
        assert_param(IS_EXB_INTERNAL_WAIT(InitStruct->Cycles.Wait));
    } else {
        assert_param(IS_EXB_EXTERNAL_WAIT(InitStruct->Cycles.Wait));
    }
    assert_param(IS_EXB_RW_SETUP_CYCLE(InitStruct->Cycles.ReadSetupCycle));
    assert_param(IS_EXB_RW_SETUP_CYCLE(InitStruct->Cycles.WriteSetupCycle));
    assert_param(IS_EXB_ALE_WAIT_CYCLE(InitStruct->Cycles.ALEWaitCycle));
    assert_param(IS_EXB_RW_RECOVERY_CYCLE(InitStruct->Cycles.ReadRecoveryCycle));
    assert_param(IS_EXB_RW_RECOVERY_CYCLE(InitStruct->Cycles.WriteRecoveryCycle));
    assert_param(IS_EXB_CS_RECOVERY_CYCLE(InitStruct->Cycles.ChipSelectRecoveryCycle));

    /* Initialize EXB ASx */
    tmp_AS = (uint32_t) InitStruct->AddrSpaceSize | ((uint32_t) InitStruct->StartAddr << 16U);
    tmp_AS |= EXB_AS_FIXED_BITS;

    /* Initialize EXB CSx */
    tmp_CS = ((uint32_t) InitStruct->Cycles.Wait << 8U) |
        ((uint32_t) InitStruct->Cycles.ReadRecoveryCycle << 24U) |
        ((uint32_t) InitStruct->Cycles.WriteRecoveryCycle << 27U) |
        ((uint32_t) InitStruct->WaitSignal << 13U) | ((uint32_t) InitStruct->WaitFunction << 12U);
    tmp_CS |= (InitStruct->Cycles.ReadSetupCycle == EXB_CYCLE_4) ?
        ((uint32_t) (InitStruct->Cycles.ReadSetupCycle - 1UL) << 16U) :
        ((uint32_t) InitStruct->Cycles.ReadSetupCycle << 16U);
    tmp_CS |= (InitStruct->Cycles.WriteSetupCycle == EXB_CYCLE_4) ?
        ((uint32_t) (InitStruct->Cycles.WriteSetupCycle - 1UL) << 18U) :
        ((uint32_t) InitStruct->Cycles.WriteSetupCycle << 18U);

    if (TSB_EXB_MOD_EXBSEL == 0U) {
        /* Set ALE wait cycle for multiplex bus */
        tmp_CS |= (InitStruct->Cycles.ALEWaitCycle == EXB_CYCLE_4) ?
            ((uint32_t) (InitStruct->Cycles.ALEWaitCycle - 1UL) << 20U) :
            ((uint32_t) InitStruct->Cycles.ALEWaitCycle << 20U);
    } else {

        /* Do nothing */
    }

    tmp_CS |= (InitStruct->Cycles.ChipSelectRecoveryCycle == EXB_CYCLE_4) ?
        ((uint32_t) (InitStruct->Cycles.ChipSelectRecoveryCycle - 1UL) << 30U) :
        ((uint32_t) InitStruct->Cycles.ChipSelectRecoveryCycle << 30U);
    tmp_CS |= (uint32_t) InitStruct->BusWidth;


    switch (ChipSelect) {
    case EXB_CS0:
        TSB_EXB->AS0 = tmp_AS;
        TSB_EXB->CS0 = tmp_CS;
        break;

    case EXB_CS1:
        TSB_EXB->AS1 = tmp_AS;
        TSB_EXB->CS1 = tmp_CS;
        break;

    case EXB_CS2:
        TSB_EXB->AS2 = tmp_AS;
        TSB_EXB->CS2 = tmp_CS;
        break;

    case EXB_CS3:
        TSB_EXB->AS3 = tmp_AS;
        TSB_EXB->CS3 = tmp_CS;
        break;

    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set the clock division setting of EXB clock output
  * @param  ClkDiv: Clock division setting 
  *   This parameter can be one of the following values:
  *   EXB_CLK_DIVISION_FSYS_2, EXB_CLK_DIVISION_FSYS_4,
  *   EXB_CLK_DIVISION_FSYS_8
  * @retval success or failure
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result EXB_SetClkOutputDivision(uint8_t ClkDiv)
{
    uint32_t tmp = 0U;
    FunctionalState state = ENABLE;
    Result retval = ERROR;
    /* Update the fsys */
    SystemCoreClockUpdate();

    /* Check the parameter */
    /* EXBCLKCTL<CLKDIV> must be set under 30MHz */
    assert_param(IS_EXB_CLK_DIVISION(ClkDiv, SystemCoreClock));

    /* Get the state of EXB clock output */
    state = EXB_GetClkOutputState();
    /* Setting of EXBCLKCTL<CLKDIV> is conducted under EXBCLKCTL<CLKEN> is "0" */
    if (state == ENABLE) {
        /* Do nothing */
    } else {
        /* Set EXBCLKCTL<CLKDIV> */
        tmp = TSB_EXB->CLKCTL;
        tmp &= EXB_CLKCTL_CLKDIV_CLEAR;
        tmp |= ClkDiv;
        TSB_EXB->CLKCTL = tmp;
        retval = SUCCESS;
    }
    return retval;
}

/**
  * @brief  Enable the EXB clock output
  * @param  None
  * @retval None
  */
void EXB_EnableClkOutput(void)
{
    uint32_t tmp = 0U;
    tmp = TSB_EXB->CLKCTL;
    tmp |= EXB_CLKCTL_CLKEN_SET;
    TSB_EXB->CLKCTL = tmp;
}

/**
  * @brief  Disable the EXB clock output
  * @param  None
  * @retval None
  */
void EXB_DisableClkOutput(void)
{
    uint32_t tmp = 0U;
    tmp = TSB_EXB->CLKCTL;
    tmp &= EXB_CLKCTL_CLKEN_CLEAR;
    TSB_EXB->CLKCTL = tmp;
}

/**
  * @brief  Get the state of EXB clock output
  * @param  None
  * @retval State of EXB clock output
  *   The value returned can be one of the following values:
  *   ENABLE or DISABLE
  */
FunctionalState EXB_GetClkOutputState(void)
{
    FunctionalState state = ENABLE;
    uint32_t tmp = 0U;
    tmp = TSB_EXB->CLKCTL;
    tmp &= EXB_CLKCTL_CLKEN_SET;

    if (tmp) {
        /* Do nothing */
    } else {
        state = DISABLE;
    }
    return state;
}

/** @} */
/* End of group EXB_Exported_Functions */

/** @} */
/* End of group EXB */

/** @} */
/* End of group TX04_Periph_Driver */
#endif                          /* defined(__TMPM461_EXB_H) */
