/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of EXB demo for the 
 *          TOSHIBA 'TMPM461' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#include <stdio.h>
#include <string.h>
#include "tmpm461_exb.h"
#include "tmpm461_gpio.h"
#include "common_uart.h"

#define SRAM_RW

#ifdef SRAM_RW
#define TEST_DATA_LEN           ((uint16_t)0x0010)
#define EXB_SRAM_START_ADDR     ((uint32_t)0x60000000)
void EXB_GPIOConfig(void);
Result rw_result = ERROR;


#endif
/*******************************************************************************
* Function Name  : main
* Description    : main function:demo for external 16-bit SRAM, multiplex bus mode
* Input          : None.
* Return         : None.
*******************************************************************************/

int main(void)
{
    uint8_t chip = EXB_CS1;
    uint8_t BusMode = EXB_BUS_MULTIPLEX;        /* Address bus and data bus are multiplexed */
    uint8_t Cycle = EXB_CYCLE_QUADRUPLE;
    /* Initial process demo */

#ifdef SRAM_RW
    uint32_t w_data[TEST_DATA_LEN] = { 0U };
    uint32_t r_data[TEST_DATA_LEN] = { 0U };
    uint16_t rw_cnt = 0U;
    uint32_t *addr = NULL;
    uint16_t i = 0U;   
#endif
    
    EXB_InitTypeDef InitStruct = { 0U };
    hardware_init(UART_RETARGET);
    common_uart_disp("EXB_Demo\r\n");
    
    InitStruct.AddrSpaceSize = EXB_128K_BYTE;
    InitStruct.StartAddr = 0x00;
    InitStruct.BusWidth = EXB_BUS_WIDTH_BIT_16;
    /* Set cycles time according to AC timing of SRAM datasheet,base clock: EXBCLK(fsys) */
    InitStruct.Cycles.Wait = EXB_WAIT_8;
    InitStruct.Cycles.ReadSetupCycle = EXB_CYCLE_2;
    InitStruct.Cycles.WriteSetupCycle = EXB_CYCLE_2;
    InitStruct.Cycles.ALEWaitCycle = EXB_CYCLE_2;
    InitStruct.Cycles.ReadRecoveryCycle = EXB_CYCLE_2;
    InitStruct.Cycles.WriteRecoveryCycle = EXB_CYCLE_2;
    InitStruct.Cycles.ChipSelectRecoveryCycle = EXB_CYCLE_2;
    InitStruct.WaitSignal = EXB_WAIT_SIGNAL_LOW;
    InitStruct.WaitFunction = EXB_WAIT_FUNCTION_INT;

#ifdef SRAM_RW
    EXB_GPIOConfig();
#endif

    EXB_SetBusMode(BusMode);
    EXB_SetBusCycleExtension(Cycle);
    EXB_Init(chip, &InitStruct);
    EXB_Enable(chip);

#ifdef SRAM_RW
    /* SRAM Read/Write demo */
    addr = (uint32_t *) (((uint32_t) InitStruct.StartAddr) | EXB_SRAM_START_ADDR);
    for (i = 0U; i < TEST_DATA_LEN; i++) {
        w_data[i] = i;
    }
    
    rw_cnt = TEST_DATA_LEN * (sizeof(w_data[0]));
    memcpy(addr, w_data, (uint32_t) rw_cnt);
    __DSB();
    memcpy(r_data, addr, (uint32_t) rw_cnt);
    /* check rw_result to see if SRAM write/read is sucessful or not */
    if (memcmp(w_data, r_data, (uint32_t) rw_cnt) == 0U) {
        rw_result = SUCCESS;
        common_uart_disp("rw_result = SUCCESS\r\n");
    } else {
        rw_result = ERROR;
        common_uart_disp("rw_result = ERROR\r\n");
    }
#endif
    while (1) {
        /* Do nothing */
    }

}

/**
  * @brief  Configure GPIO to EXB
  * @param  None
  * @retval None
  */


void EXB_GPIOConfig(void)
{
    /* PK PORT[2:5] FR1 as RD WR BELL BELH */
    GPIO_SetOutputEnableReg(GPIO_PK, 0x3CU, ENABLE);
    GPIO_EnableFuncReg(GPIO_PK, GPIO_FUNC_REG_1, 0x3CU);
    GPIO_SetInputEnableReg(GPIO_PK, 0x3CU, ENABLE);
    GPIO_SetPullUp(GPIO_PK, 0x3CU, DISABLE);

    /* PC PORT[0:7] FR2 address bus(A16-A23) */
    GPIO_SetOutputEnableReg(GPIO_PC, 0xFFU, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, 0xFFU);
    GPIO_SetInputEnableReg(GPIO_PC, 0xFFU, ENABLE);

    /* PD PORT[0:15] FR1 as DATA(D0-D15)/AD0-AD15  bus */
    GPIO_SetOutputEnableReg(GPIO_PD, 0xFFFFU, ENABLE);
    GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_1, 0xFFFFU);
    GPIO_SetInputEnableReg(GPIO_PD, 0xFFFFU, ENABLE);

    /* PC PORT[8] PORT[10] FR1 as ALE CS1 */
    GPIO_SetOutputEnableReg(GPIO_PC, 0x500U, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_1, 0x500U);
    GPIO_SetInputEnableReg(GPIO_PC, 0x500U, ENABLE);
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
