/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of IGBT One PPG for the TOSHIBA 'TMPM461' 
 *          Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Global variable define ----------------------------------------------------*/
WorkState counter_state = BUSY;
Result cancel_result = ERROR;
IGBT_InitTypeDef myIGBT;
IGBT_EMGStateTypeDef EMGState = { IGBT_EMG_INPUT_LOW, IGBT_EMG_NORMAL };

/**
  * @brief  main function to run IGBT One PPG demo
  * @param  None.
  * @retval result to retun to system.
  */
int main(void)
{
    /* LED initialization */
    LED_Init();
    LED_On(LED0);

    myIGBT.StartMode = IGBT_FALLING_TRG_START;  /* IGBT trigger start: falling edge start and active level is "Low" */
    myIGBT.OperationMode = IGBT_CONTINUOUS_OUTPUT;      /* IGBT operation: continuous operation */
    myIGBT.CntStopState = IGBT_OUTPUT_INACTIVE; /* IGBT stopping status: initial output satus and counter */
    myIGBT.ActiveAcceptTrg = DISABLE;   /* Trigger edge accept mode: Don't accept trigger during active level */
    myIGBT.INTPeriod = IGBT_INT_PERIOD_1;       /* Interrupt cycle: Every one cycle */
    myIGBT.ClkDiv = IGBT_CLK_DIV_2;     /* For M461:fperiph = fc = 16MHz*6 = 96MHz, T0 = fperiph = 96MHz, figbt = T0/2 = 48MHz */
    myIGBT.Output0Init = IGBT_OUTPUT_HIGH_ACTIVE;       /* MTOUT00 initial state is Low, and active level is High */
    myIGBT.Output1Init = IGBT_OUTPUT_DISABLE;   /* Disable MTOUT10 output */
    myIGBT.TrgDenoiseDiv = IGBT_DENOISE_DIV_240;        /* Trigger input noise elimination time: 240/fsys */
    myIGBT.Output0ActiveTiming = 1U;
    myIGBT.Output0InactiveTiming = 1U + IGBT_PPG_PERIOD_50US / 2;
    myIGBT.Period = IGBT_PPG_PERIOD_50US;       /* Period: 50us */
    myIGBT.EMGFunction = IGBT_EMG_OUTPUT_HIZ;   /* The polarity of MTOUT0x at EMG protection: High-impedance */
    myIGBT.EMGDenoiseDiv = IGBT_DENOISE_DIV_240;        /* EMG input noise elimination time: 240/fsys */

    IGBT_IO_Configuration(IGBT0, &myIGBT);

    IGBT_Enable(IGBT0);
    NVIC_EnableIRQ(INTMTEMG0_IRQn);     /* Enable IGBT and EMG interrupt */

    do {                        /* If the timer is still running, wait until it stops */
        counter_state = IGBT_GetCntState(IGBT0);
    } while (counter_state == BUSY);
    do {                        /* Cancel the EMG protection state */
        cancel_result = IGBT_CancelEMGState(IGBT0);
    } while (cancel_result == ERROR);

    IGBT_Init(IGBT0, &myIGBT);

    IGBT_SetSWRunState(IGBT0, IGBT_RUN);        /* Prepare to run IGBT timer */

    while (1) {

        EMGState = IGBT_GetEMGState(IGBT0);     /* Get EMG protection state to check EMG state */

        if (EMGState.EMGProtectState == IGBT_EMG_PROTECT) {     /* If EMG protection is on going */
            do {
                counter_state = IGBT_GetCntState(IGBT0);
            } while (counter_state == BUSY);
            do {                /* Cancel the EMG protection state */
                cancel_result = IGBT_CancelEMGState(IGBT0);
            } while (cancel_result == ERROR);

            LED_Off(LED1);      /* Turn off the LED and prepare to run again */
            IGBT_Init(IGBT0, &myIGBT);

            IGBT_SetSWRunState(IGBT0, IGBT_RUN);
        } else {
            /* Do nothing */
        }
    }
}

/**
  * @brief  IRQ for EMG protection
  * @param  None.
  * @retval None.
  */
void INTMTEMG0_IRQHandler(void)
{
    LED_On(LED1);               /* If EMG protection, turn on the LED and stop the timer */
    IGBT_SetSWRunState(IGBT0, IGBT_STOP);
}

/**
  * @brief  IGBT port initialization
  * @param  IGBTx: IGBT channel.
  *         InitTypeDef: initialization structure.
  * @retval None.
  */
void IGBT_IO_Configuration(TSB_MT_TypeDef * IGBTx, IGBT_InitTypeDef * InitTypeDef)
{
    if (IGBTx == IGBT0) {
        /* Check whether the trigger is used to start */
        if ((InitTypeDef->StartMode != IGBT_CMD_START) ||
            (InitTypeDef->StartMode != IGBT_CMD_START_NO_START_INT)) {
            GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_13, ENABLE);        /* Enable PD13 input */
            GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_4, GPIO_BIT_13);   /* Set PD13 as MT0IN */
        } else {
            GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_13, DISABLE);
            GPIO_DisableFuncReg(GPIO_PD, GPIO_FUNC_REG_4, GPIO_BIT_13);
        }
        /* Enable ouput0 or not */
        if (InitTypeDef->Output0Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_14, ENABLE);       /* Enable PD14 output */
            GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_4, GPIO_BIT_14);   /* Set PD14 as MT0OUT0 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_14, DISABLE);
            GPIO_DisableFuncReg(GPIO_PD, GPIO_FUNC_REG_4, GPIO_BIT_14);
        }
        /* Enable ouput1 or not */
        if (InitTypeDef->Output1Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_15, ENABLE);       /* Enable PD15 output */
            GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_4, GPIO_BIT_15);   /* Set PD15 as MT0OUT1 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_15, DISABLE);
            GPIO_DisableFuncReg(GPIO_PD, GPIO_FUNC_REG_4, GPIO_BIT_15);
        }

        GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_12, ENABLE);    /* Enable PD12 input */
        GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_4, GPIO_BIT_12);       /* Set PD12 as GEMG0 */
    }

    if (IGBTx == IGBT1) {
        /* Check whether the trigger is used to start */
        if ((InitTypeDef->StartMode != IGBT_CMD_START) ||
            (InitTypeDef->StartMode != IGBT_CMD_START_NO_START_INT)) {
            GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_5, ENABLE);        /* Enable PC5 input */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_3, GPIO_BIT_5);   /* Set PC5 as MT1IN */
        } else {
            GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_5, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_3, GPIO_BIT_5);
        }
        /* Enable ouput0 or not */
        if (InitTypeDef->Output0Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_6, ENABLE);       /* Enable PC6 output */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_3, GPIO_BIT_6);   /* Set PC6 as MT1OUT0 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_6, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_3, GPIO_BIT_6);
        }
        /* Enable ouput1 or not */
        if (InitTypeDef->Output1Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_7, ENABLE);       /* Enable PC7 output */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_3, GPIO_BIT_7);   /* Set PC7 as MT1OUT1 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_7, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_3, GPIO_BIT_7);
        }

        GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_4, ENABLE);    /* Enable PC4 input */
        GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_3, GPIO_BIT_4);       /* Set PC4 as GEMG1 */
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif                          /* DEBUG */
